@@ -20,6 +20,40 @@ mod allocator;
 mod dispatcher;
 mod logger;
 
+// For crate-type="cdylib".
+#[cfg(not(wasi_exec_model_reactor))]
+#[macro_export]
+macro_rules! main {
+    ($code:block) => {
+        #[cfg(target_os = "wasi")]
+        extern "C" {
+            fn __wasm_call_ctors();
+        }
+
+        #[no_mangle]
+        pub extern "C" fn _initialize() {
+            #[cfg(target_os = "wasi")]
+            unsafe {
+                __wasm_call_ctors();
+            }
+
+            $code;
+        }
+    };
+}
+
+// For crate-type="bin" with RUSTFLAGS="-Z wasi-exec-model=reactor".
+#[cfg(wasi_exec_model_reactor)]
+#[macro_export]
+macro_rules! main {
+    ($code:block) => {
+        pub fn main() -> Result<(), Box<dyn std::error::Error>> {
+            $code;
+            Ok(())
+        }
+    };
+}
+
 pub fn set_log_level(level: types::LogLevel) {
     logger::set_log_level(level);
 }
@@ -37,4 +71,4 @@ pub fn set_http_context(callback: types::NewHttpContext) {
 }
 
 #[no_mangle]
-pub extern "C" fn proxy_abi_version_0_1_0() {}
+pub extern "C" fn proxy_abi_version_0_2_1() {}
